#/* Pdx/Base.py  -*-python-*-
# * Copyright (C) 1999,2003  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */
"""
NAME:
    Base.py  

SYNOPSIS:
    import Base
    class <backend>(Base.Base)
    
DESCRIPTION:
    Pdx parser.  Inherited by format-specific backends.

OPTIONS:
    -h,--help        this message
    -v,--version     version
    -d,--debug       turn on debug messages

EXAMPLES:
    1. As library
        class Html (Base.Base):
        ...
"""

__version__='1.4'

import sys,os,types,time,re,string
from types import *
#import pyperl
#p=pyperl.PyPerl()
#revisions:
#1999-09-24  Harry George    reimplement from perl version

def debug(ftn,txt):
    sys.stdout.write("Pdx.Base."+ftn+":"+txt+"\n")
    sys.stdout.flush()
#===globals============================

#============================
class Base:
    classname="Pdx.Base"
    #---block-style re's-------------
    back_pat=re.compile(r'^=back')
    begin_pat=re.compile(r'^=begin\s+([A-Za-z0-9\_]+)')
    center_pat=re.compile(r'^=center')
    column_pat=re.compile(r'^=columns\s+(\d+)')
    columns_pat=re.compile(r'^=columns')    
    comment_pat=re.compile(r'^\s*#')
    cut_pat=re.compile(r'^=cut')
    cfg_pat=re.compile(r'^=cfg')
    def1_pat=re.compile(r'^([a-z0-9_/]+)\s*(\+?==?)\s*(.+)')
    def2_pat=re.compile(r'^=def\s+([a-z0-9_/]+)\s*(\+?=?)\s*(.+)')
    end_pat=re.compile(r'^\s*=end')
    equal_pat=re.compile(r'^=')
    expand_pat=re.compile(r'@([a-z0-9_/]+)@')
    for_pat=re.compile(r'^=for\s+(\S+)\s+(.+)')
    head_pat=re.compile(r'^=head(\d)\s+(.+)')
    include_pat=re.compile(r'^=include\s+(\S+)')
    include_verbatim_pat=re.compile(r'^=include_verbatim\s+(\S+)')
    include_for_pat=re.compile(r'^=include_for\s+(\S+)\s+(\S+)')
    include_raw_pat=re.compile(r'^=include_raw\s+(\S+)\s+(\S+)')
    item_pat=re.compile(r'^\s*=item\s+(.+)')
    list_pat=re.compile(r'^\s*=list\s+([\*1aAiIscN\[])')
    out_pat=re.compile(r'^\s*[^=]')
    over_pat=re.compile(r'^=over\s+(\d+)')
    para_pat=re.compile(r'^\s*$')
    pathpush_pat=re.compile(r'^=pathpush\s+(\S+)')
    pathpop_pat=re.compile(r'^=pathpop')
    preamble_begin_for_pat=re.compile(r'^=preamble_begin_for\s+(\S+)')
    preamble_end_for_pat  =re.compile(r'^=preamble_end_for\s+(\S+)')
    preamble_append_for_pat=re.compile(r'^=preamble_append_for\s+(\S+)')
    postamble_for_pat=re.compile(r'^=postamble_for\s+(\S+)')
    quote_pat=re.compile(r'^=quote')
    row_pat=re.compile(r'^=row (.+)')
    row_amp_pat=re.compile(r'\s&\s')
    slide_pat=re.compile(r'=slide\s+(\S+)')
    table_pat=re.compile(r'^=table(.*)')
    table_cell_pat=re.compile(r'^=cell')
    texmath_pat=re.compile(r'^=texmath')
    verbatim_pat=re.compile(r'^=verbatim')
    #---in-line re's--------------
    hr_pat=re.compile(r'<HR>')       
    break_pat=re.compile(r'<BR>')
    X_pat=re.compile(r'\bX<([^>]+)>')
    B_pat=re.compile(r'\bB<([^>]+)>')
    I_pat=re.compile(r'\bI<([^>]+)>')
    C_pat=re.compile(r'\bC<([^>]+)>')
    U_pat=re.compile(r'\bU<([^>]+)>')
    N_pat=re.compile(r'\bN<([^>]+)>')
    L_pat=re.compile(r'\bL<"([^"]+)","([^"]+)">')
    L_hash_pat=re.compile(r'\bL<"#([^"]+)","([^"]+)">')
    L_cgi_pat=re.compile(r'\bL<"([^\?]+)\?([^"]+)","([^"]+)">')
    lt_pat=re.compile(r'E<lt>')
    gt_pat=re.compile(r'E<gt>')
    amp_pat=re.compile(r'E<amp>')
    quot_pat=re.compile(r'E<quot>')
    lb_pat=re.compile(r'E<lb>')
    vertbar_pat=re.compile(r'\|')


    #--------------------
    def __init__(self):
        ftn="init"
        self.driver=None
        self.driverfile=None
        self.extension='.txt'
        self.preamble_begin=[]
        self.preamble_append=[]
        self.preamble_end=[]
        self.postamble=[]
        self.lines=[]
        self.ndx=0
        self._pass=0
        self.data={}
        self._toc=[]
        self._index=[]
        self._refs={}
        self._ref_cnt=0
        self._errcnt=0
        self._labelnumber=0
        self.headstack=[]
        self.liststack=[]
        self.startdir=os.getcwd()
        self.pathstack=['.']  #initialize to current dir
        self.table_numcols=0
        self.table_colwidths=[]
        self.table_cells=[]

    #----------------
    def __del__(self):
        pass
    #------------------------
    def close_files(self):
        if self.driverfile != sys.stdout:
            self.driverfile.close()

    #--------------------------
    def err(self,txt):
        print "%(filename)s line %(ndx)d: ERROR %(txt)s" % \
        {'filename':self.data['_filename'],'ndx':self.ndx,'txt':txt}
        self._errcnt=self._errcnt+1

    #------------------------
    def fatal(self,txt):  
        print "%(filename)s line %(ndx)d: FATAL %(txt)s" % \
              {'filename':self.data['_filename'],'ndx':self.ndx,'txt':txt}
        raise SystemExit

    #-----------------------
    def genlabel(self):    
        self._labelnumber = self._labelnumber+1
        return 'L%03.3d' % self._labelnumber

    #----------------------
    def get_date(self):
        (year,mon,day,hr,min,sec,wkday,day2,dst)=time.localtime(time.time())
        if self.data['date']=="":
            self.data['date']="%04.4d-%02.2d-%02.2d" % (year,mon,day)
        if self.data['validuntil']=="":
            self.data['validuntil']="%04.4d-%02.2d-%02.2d" % (year,12,31)

    #--------------------
    def morelines(self):
        return self.ndx < len(self.lines)

    #--------------------
    def nextline(self):
       ftn="nextline"
       try: 
           line=self.lines[self.ndx]
           self.ndx=self.ndx+1
       except:
           self.fatal("cannot get next line")
       #debug(ftn,"pre-expand:%i:%s" % (self.ndx,line))
       if self.data['expand_p'] == "1":
           while(1):
               m=self.expand_pat.search(line)
               if not m: break
               key=m.group(1)
               if self.data.has_key(key):
                   tmp=self.data[key]
                   if not isinstance(tmp,types.StringType):
                       value=`tmp`
                   else:
                       value=tmp                   
               else:
                   value=""
               line = string.join([line[:m.start(0)],value,line[m.end(0):]],'')
       #debug(ftn,"post-expand:%i:%s" % (self.ndx,line))
       return line

    #-------------------
    def open_files(self):
        ftn="Base.open_files"
        #---input---
        if self.data['_stdin_p'] == "1":
            self.lines=sys.stdin.readlines()
        else:
            filename=self.data['_filename']
            if not os.path.exists(filename):
                self.fatal("cannot find "+ filename)
            localpath,localbase=os.path.split(filename)
            self.do_pathpush(localpath)
            file=open(filename,"r")
            self.lines=file.readlines()
            file.close()
            
        #---output---
        if self.data['_stdout_p'] == "1":
            self.driverfile=sys.stdout
        else:
            outfile=os.path.join(self.data['_dirname'],
                                 self.data['_basename']+self.extension)
            self.driverfile=open(outfile,"w")
            #debug(ftn,"filename=%s, file=%s" % (outfile,`self.driverfile`))
    #--------------------
    def out(self,line):
        #---any '<' or '_' on the line?---
        if (string.find(line,'<') != -1) or (string.find(line,'_') != -1):
            #---indexing---
            m=self.X_pat.search(line);
            if m:
                tmp1=line[:m.start(0)]
                tmp2=self.driver_xref(m.group(1))
                tmp3=line[m.end(0):]
                line=tmp1+tmp2+tmp3
            #---driver-unique---
            if self._pass==1: return
            line=self.driver_line(line)
        if self._pass==1: return        
        self.rawout(line)
    #-------------------
    def parse_args(self,args):
        ftn="parse_args"
        argndx=0
        while argndx<len(args):
            arg=args[argndx]
            argndx=argndx+1
            #debug(ftn,"arg="+arg)
            if arg=='--stdout':
                self.data['_stdout_p']="1"
            elif arg=='--stdin':
                self.data['_stdin_p']="1"
            elif arg=='--index':
                self.data['index_p']="1"
            elif arg=='--toc':
                self.data['toc_p']="1"
            elif arg=='--notoc':
                self.data['toc_p']="0"
            elif arg=='--slides':
                self.data['slides_p']="0"
                
            elif re.match(r'^[A-Za-z0-9\/\-\.\_]+$',arg):
                self.data['_filename']=arg
                self.data['_dirname']=os.path.dirname(arg)
                fullbase=os.path.basename(arg)
                #debug(ftn,"fullbase="+fullbase)
                (base,ext)=os.path.splitext(fullbase)
                #debug(ftn,"base="+base+" ext="+ext)
                self.data['_basename']=base
            else:
                self.fatal("unexpected arg:"+arg)

        #---check validity of settings---
        if not self.data.has_key('_basename'):
            self.fatal("must provide filename")

        #---set defaults---
        if not self.data.has_key('_stdout_p'):
           self.data['_stdout_p']="0" 
        if not self.data.has_key('_stdin_p'):
           self.data['_stdin_p']="0" 
        if not self.data.has_key('_force'):
           self.data['_force']="0" 
        if not self.data.has_key('toc_p'):
           self.data['toc_p']="1" 
        if not self.data.has_key('index_p'):
           self.data['index_p']="0" 
        if not self.data.has_key('show_cut_p'):
           self.data['show_cut_p']="0" 
        if not self.data.has_key('numbered_heads_p'):
           self.data['numbered_heads_p']="0" 
        if not self.data.has_key('expand_p'):
           self.data['expand_p']="1" 
        if not self.data.has_key('slides_p'):
           self.data['slides_p']="0" 

        if not self.data.has_key('date'):
           self.data['date']="" 

        if not self.data.has_key('validuntil'):
           self.data['validuntil']="" 


    #-------------------
    def parse(self,args):
        ftn = "parse"
        self.parse_args(args)
        self.get_date()
        self.open_files()
        self.initial_data=self.data
        for p in (1,2):
            self._pass=p
            #debug(ftn,"pass="+`self._pass`)
            if self._errcnt>0:
               self.fatal(self._errcnt + " error(s) in first pass")
            if self._pass==2: self.reset()
            if self._pass==2: self.write_preamble()
            self.process_file()
            if self._pass==2: self.write_postamble()

        self.close_files()
        return 1

    #----------------------
    def process_file(self):
        ftn="process_file"
        #debug(ftn,"begin")
        while self.morelines():
            line=self.nextline()
            #debug(ftn,"line="+line)
            if self.end_pat.search(line):
                self.ndx=self.ndx-1; return
            if self.comment_pat.search(line):
                continue
            if self.para_pat.match(line):
                self.do_para(); continue
            if self.cfg_pat.search(line):
                self.do_cfg(); continue

            m=self.def2_pat.search(line)
            if m:
                self.do_def(m.group(1),m.group(2),m.group(3)); continue

            m=self.item_pat.search(line)
            if m:
                #---let do_list see the item line---
                self.ndx=self.ndx-1
                return

            #---normal lists---
            m=self.list_pat.search(line)
            if m:
                self.do_list(m.group(1)); continue


            #---heads---
            m=self.head_pat.search(line)
            if m:
                self.do_head(m.group(1),m.group(2)); continue

            #---various includes---
            m=self.verbatim_pat.search(line)
            if m:
                self.do_verbatim(); continue

            m=self.pathpush_pat.search(line)
            if m:
                self.do_pathpush(m.group(1)); continue

            m=self.pathpop_pat.search(line)
            if m:
                self.do_pathpop(); continue

            m=self.include_pat.search(line)
            if m:
                self.do_include(m.group(1)); continue

            m=self.include_verbatim_pat.search(line)
            if m:
                self.do_include_verbatim(m.group(1)); continue

            m=self.include_raw_pat.search(line)
            if m:
                self.do_include_raw(m.group(1),m.group(2)); continue

            m=self.include_for_pat.search(line)
            if m:
                self.do_include_for(m.group(1),m.group(2)); continue

            m=self.for_pat.search(line)
            if m:
                self.do_for(m.group(1),m.group(2)); continue

            m=self.begin_pat.search(line)
            if m:
                self.do_begin(m.group(1)); continue

            #---assorted features---
            m=self.table_pat.search(line)
            if m:
                self.do_table(m);continue

            m=self.slide_pat.search(line)
            if m:
                self.do_slide(m.group(1));continue

            m=self.center_pat.search(line)
            if m:
                self.driver_center(); continue

            m=self.quote_pat.search(line)
            if m:
                self.driver_quote(); continue

            m=self.texmath_pat.search(line)
            if m:
                self.do_texmath(); continue

            #---pod-style over...back lists---
            m=self.over_pat.search(line)
            if m:
                self.do_over(); continue

            m=self.back_pat.search(line)
            if m:
                self.ndx=self.ndx-1; return

            #---pre/post ambles----
            m=self.preamble_begin_for_pat.search(line)
            if m:
                self.do_preamble_begin_for(m.group(1)); continue

            m=self.preamble_append_for_pat.search(line)
            if m:
                self.do_preamble_append_for(m.group(1)); continue

            m=self.preamble_end_for_pat.search(line)
            if m:
                self.do_preamble_end_for(m.group(1)); continue

            m=self.postamble_for_pat.search(line)
            if m:
                self.do_postamble_for(m.group(1)); continue

            m=self.cut_pat.search(line)
            if m:
                self.do_cut(); continue

            #---simple case---
            if self.out_pat.search(line):
                #---optional leading spaces, then non-"="---
                #---so output it---
                self.out(line); continue
                
            #---hmmm, couldn't recognize it---
            self.fatal("unknown line type:"+line)

    #--------------------
    def rawout(self,line):
        if self._pass==1: return
        self.driverfile.write(line)
    #--------------------
    def reset(self):
        self.ndx=0
        self._labelnumber=0
        self.headstack=[]
        self.liststack=[]
        self.pathstack=['.']
        self.driver_reset()
    #--------------------
    def write_preamble(self):
        ftn="write_preamble"
        #debug(ftn,"begin")
        for line in self.preamble_begin:
            #debug(ftn,"pre="+line)
            self.rawout(line)
        for line in self.preamble_append:
            self.rawout(line)
        for line in self.preamble_end:
            self.rawout(line)
        if self.driver=='pypt':
            self.driver_slide_begin('Default')
        if self.driver!='docbook' and self.data['toc_p']=="1":
            self.driver_toc()
    #-------------------
    def write_postamble(self):
        ftn="write_postamble"
        #debug(ftn,"begin")
        if self.driver=='pypt':
            self.driver_slide_end()
        if self.driver=='docbook' and self.data['toc_p']=="1":
            self.driver_head_end()
            self.driver_toc()
        if self.data['index_p']=="1":
            self.driver_index()
        for line in self.postamble:
            self.rawout(line)


#===processing======================
    #----------------
    def do_begin(self,driver):
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line): break
            if driver == self.driver:
                self.out(line)

    #-------------
    def do_cfg(self):
        ftn = "do_cfg"
        #debug(ftn,"begin")
        name=sep=value=None
        while self.morelines():
            line=self.nextline()
            line=string.strip(line)
            if self.end_pat.search(line):break
            if line=="" or self.comment_pat.search(line): continue
            m=self.def1_pat.search(line)
            if m != None:
                name,sep,value=m.groups()
                #debug(ftn,"name/value pair:"+name+"="+value)
            else:
                self.err("unknown cfg:"+line)
            self.do_def(name,sep,value)

    #----------------
    def do_cut(self):
        lines=[]
        while self.morelines():
            line=self.nextline()
            if self.equal_pat.search(line): break
            lines.append(line)
        if self.data.has_key('show_cut') and self.data['show_cut']:
            self.driver_verbatim(lines)
    #-------------------
    def do_def(self,name,sep,value):
        ftn="do_def"
        tmp=None
        value=str(value)  #make sure it is a string
        if re.search(r'^_',name):
            self.err("cannot define '_' names")
        if sep=='=':
            #---preprocess the line---
            self.data[name]=self.driver_line(value)
        elif sep == '==':
            #---use line as-is---
            self.data[name]=value
        elif sep== '+=':
            #---append preprocessed to existing data---
            if not self.data.has_key(name): self.data[name]=""
            self.data[name]=self.data[name]+self.driver_line(value)
        elif sep == '+==':
            #---append as-is to existing data---
            if not self.data.has_key(name): self.data[name]=""
            self.data[name]=self.data[name]+value

    #------------------
    def do_for(self,driver,line):
        if self.driver==driver:
            self.rawout(line)
    #-------------------
    def do_head(self,level,name):
        label=self.genlabel()
        if self.data['numbered_heads_p'] == "1":
            self.driver_head_numbered(label,level,name)
        else:
            self.driver_head(label,level,name)

    #------------------
    def do_include_getdata(self,filename):
        ftn="do_include_getdata"
        #debug(ftn,"pathstack=%s" % self.pathstack)
        (newdir,leaf)=os.path.split(filename)
        #debug(ftn,"newdir=%s, leaf=%s" % (newdir,leaf))
        
        fullname=self.startdir
        for segment in self.pathstack:
            fullname=os.path.join(fullname,segment)
        fullname=os.path.join(fullname,newdir,leaf)        
        #debug(ftn,"fullname=%s" % fullname)
        
        try:
            file=open(fullname,"r")
            lines=file.readlines()
            file.close()
        except:
            self.fatal("cannot open "+fullname)

        return lines,newdir

    #------------------
    def do_include(self,filename):
        ftn = "do_include"
        if self._pass==2: return

        #---backup to overwrite the include stmt---
        self.ndx=self.ndx-1
        newlines,newdir=self.do_include_getdata(filename)
        if newdir:
            lines=["=pathpush %s\n" % newdir]+newlines+["=pathpop\n"]
        else:
            lines=newlines
        self.lines[self.ndx:self.ndx+1]=lines
    #------------------
    def do_include_for(self,driver,filename):
        if self.driver == driver:
            self.do_include(filename)

    #------------------
    def do_include_raw(self,driver,filename):
        if self.driver == driver:
            newlines,newdir=self.do_include_getdata(filename)
            for line in newlines:
                self.out(line)
    #------------------
    def do_include_verbatim(self,filename):
        ftn="do_include_verbatim"
        if self._pass==2: return

        #---backup to overwrite the include stmt---
        #---ignore newdir, because include_verbatim is the end of the line--
        self.ndx=self.ndx-1
        newlines,newdir=self.do_include_getdata(filename)
        lines=["=verbatim\n"]
        lines.extend(newlines)
        lines.append("=end verbatim\n")
        self.lines[self.ndx:self.ndx+1]=lines
    #----------------
    def do_list(self,marker):
        self.driver_list(marker)

    #----------------
    def do_over(self):
        #---perl-style---
        self.driver_list('[')

    #-------------------
    def do_para(self):
        self.driver_para()

    #-------------------
    def do_pathpush(self,name):
        if len(name) > 0:
            self.pathstack.append(name)

    #-------------------
    def do_pathpop(self):
        if len(self.pathstack) >0:
            self.pathstack.pop()

    #--------------------
    def do_preamble_begin_for(self,name):
        ftn = "do_preamble_begin_for"
        #debug(ftn,"driver="+self.driver+", for_name="+name)
        lines=[]
        while self.morelines():
           line=self.nextline()
           if self.end_pat.search(line): break
           lines.append(line)
        #for line in lines: debug(ftn,line)
        if self.driver == name and self.preamble_begin==[]:
           self.preamble_begin=lines
    #--------------------
    def do_preamble_append_for(self,name):
       lines=[]
       while self.morelines():
           line=self.nextline()
           if self.end_pat.search(line): break
           lines.append(line)
       if self.driver == name:
           self.preamble_append=lines

    #--------------------
    def do_preamble_end_for(self,name):
       lines=[]
       while self.morelines():
           line=self.nextline()
           if self.end_pat.search(line): break
           lines.append(line)
       if self.driver == name and self.preamble_end==[]:
           self.preamble_end=lines

    #--------------------
    def do_postamble_for(self,name):
       lines=[]
       while self.morelines():
           line=self.nextline()
           if self.end_pat.search(line): break
           lines.append(line)
       if self.driver == name and self.postamble==[]:
           self.postamble=lines

    #-------------------
    def do_slide(self,title):
        ftn="Base.do_slide"
        slide_p=self.data.get('slide_p',None)
        if slide_p=="1":
            self.driver_slide_begin(title)
        self.process_file()
        self.nextline() # past =end
        if slide_p=="1":
            self.driver_slide_end()

    #-------------------
    def do_table(self,m):
        ftn="Base.do_table"
        #debug(ftn,"groups=%s" % `m.groups()`)        
        if not m.groups() or not m.group(1):
            self.do_table0("")
        else:
            parts=string.split(m.group(1),';')
            if len(parts)==1:
                self.do_table0(m.group(1))
            elif len(parts)==3:
                colwidths=string.strip(parts[0])[1:-1]
                grid=string.strip(parts[1])
                caption=string.strip(parts[2])
                self.do_table1(colwidths,grid,caption)
            else:
                err(ftn,"invalid table entry")
            
    #-------------------
    def do_table0(self,caption):
        """default style"""
        ftn="Base.do_table0"
        self.table_grid_p=1
        self.table_numcols=0
        self.table_colwidths=[]
        
        #---get numcols from firstrow---
        first_row_p=self.first_row_p=1
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line): break
            if self.comment_pat.search(line): continue
            if self.table_cell_pat.search(line):
                self.process_file()
                continue
            m=self.row_pat.search(line)
            if m:
                self.do_table_row(m.group(1),first_row_p,caption)
            first_row_p=self.first_row_p=0
        self.driver_table_end(caption)
    #-------------------
    def do_table1(self,colwidths,grid,caption):
        """with percent field sizes, and grid/nogrid"""
        widths=[]
        for x in re.split(',',colwidths):
            widths.append(int(x))
        self.table_colwidths=widths
        self.table_numcols=len(widths)
        if grid == 'grid':
            self.table_grid_p=1
        else:
            self.table_grid_p=0
        self.driver_table_begin(caption)
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line): break
            if self.comment_pat.search(line): continue
            m=self.row_pat.search(line)
            if m:
                self.do_table_row(m.group(1))
        self.driver_table_end(caption)

    #-------------------
    def do_table_row(self,line,first_row_p=0,caption=None):
        ftn="do_table_row"
        #debug(ftn,"begin")
        self.table_col_ndx=0
        
        if first_row_p==1:
            #---first line; get all cells before processing---
            while self.morelines():
                tmpline=self.nextline()
                tmpline=tmpline.rstrip()
                if self.end_pat.search(tmpline) \
                or self.row_pat.search(tmpline) \
                or self.comment_pat.search(tmpline):
                    #---done with row---
                    self.ndx=self.ndx-1
                    break
                else:
                    #---line continuation---
                    line=line+tmpline

            cells=self.table_cells=self.row_amp_pat.split(line)

            if True or self.driver=='latex' or self.driver=='rest':
                #---need numcols and widths---
                self.table_numcols=len(cells)
                widths=[]
                linelen=float(len(line))
                for cell in cells:
                    cell_len=float(len(cell))
                    percent=int(cell_len/linelen*100.0)
                    widths.append(percent)
                self.table_colwidths=widths
            self.driver_table_begin(caption)
            self.do_table_row_out(cells)
            
        else:
            #---normal lines; process cells on the fly---
            self.driver_table_row_begin()
            while self.morelines():
                tmpline=self.nextline()
                tmpline=tmpline.rstrip()
                if self.end_pat.search(tmpline) \
                or self.row_pat.search(tmpline) \
                or self.comment_pat.search(tmpline):
                    #---done with row; do remaining items---
                    self.ndx=self.ndx-1
                    if line:
                        cells=self.row_amp_pat.split(line)
                        self.do_table_cells(cells)
                    break
                elif self.table_cell_pat.search(tmpline):
                    #---process the line as known so far---
                    cells=self.table_cells=self.row_amp_pat.split(line)
                    self.do_table_cells(cells)
                    #---do the embedded cell---
                    self.do_table_cell_begin()
                    self.process_file()
                    self.ndx=self.ndx+1  #past =end cell
                    self.do_table_cell_end()
                    #---restart line---
                    line=''
                else:
                    #---line continuation---
                    line=line+tmpline
            self.driver_table_row_end()

    #--------------------------
    def do_table_row_out(self,cells):
        self.colwidth_ndx=0
        self.driver_table_row_begin()
        self.do_table_cells(cells)
        self.driver_table_row_end()
        
    #--------------------------
    def do_table_cells(self,cells):
        for i,cell in enumerate(cells):
            self.table_col_ndx=i
            self.do_table_cell_begin()
            self.out(cell)
            self.do_table_cell_end()
        
    #--------------------------
    def do_table_cell_begin(self):
        if self.table_colwidths:
            width=self.table_colwidths[self.table_col_ndx]
        else:
            width=None
        self.driver_table_cell_begin(width)
        self.table_col_ndx+=1
        
    #---------------------------
    def do_table_cell_end(self):
        self.driver_table_cell_end()
        
    #-------------------
    def do_texmath(self):
        #---save and turn off expansion---
        expand_state=self.data['expand_p']
        self.data['expand_p']="0"
        #---collect lines---
        lines=[]
        while 1:
            line=self.nextline()
            if self.end_pat.search(line): break
            lines.append(line)
        #---restore expansion state---
        self.data['expand_p']=expand_state
        #---output lines---
        self.driver_texmath(lines)
    #--------------------
    def do_texmath_png(self,filename,lines):
        """Convert tex to png, using latex, dvips, pstoimg, netpbm"""
        if self._pass==1: return
        file=open(filename+".tex","w");
        file.write("""
\\documentclass[fleqn,12pt]{article}
\\pagestyle{empty}
\\setlength{\mathindent}{0em}
\\begin{document}
\\begin{Large}
""")

        for line in lines: file.write(line);
        file.write("\\end{Large}\n\\end{document}\n")
        file.close()

        debugging=0
        if debugging:
            os.system("latex %s.tex" % filename)
            os.system("dvips -o %s.ps %s.dvi" % (filename,filename))
            os.system("pstoimg -crop a -type png -transparent %s.ps" % filename)
        else:
            #---with interactive outputs turned off---
            os.system("latex %s.tex > /dev/null" % filename)
            os.system("dvips -o %s.ps %s.dvi > /dev/null 2> /dev/null" % \
                      (filename,filename))
            os.system(("pstoimg -crop a -type png -transparent %s.ps  "
                       "> /dev/null 2> /dev/null") % filename)
            os.system("rm "
                   +filename+".tex "
                   +filename+".dvi "
                   +filename+".aux "
                   +filename+".log "
                   +filename+".ps  "
                   +"  > /dev/null 2> /dev/null")

    #------------------
    def do_verbatim(self):
        ftn="do_verbatim"
        #---save and turn off expansion---
        expand_state=self.data['expand_p']
        self.data['expand_p']="0"
        #---collect lines---
        lines=[]
        while 1:
            line=self.nextline()
            if self.end_pat.search(line):
                #---must be on left margin---
                if line[0]=='=':
                    break
            lines.append(line)
        #debug(ftn,"lines="+`lines`)
        #---restore expansion state---
        self.data['expand_p']=expand_state
        #---output lines---
        self.driver_verbatim(lines)

    #------------------
    def do_verbatim_pod(self):
        ftn = "do_verbatim_pod"
        debug(ftn,"begin")
        #---save and turn off expansion---
        expand_state=self.data['expand_p']
        self.data['expand_p']="0"
        #---collect lines---
        lines=[]
        while 1:
            line=self.nextline()
            if re.search(r'^\S',line):
                self.ndx=self.ndx-1
                break
            lines.append(line)
        #---restore expansion state---
        self.data['expand_p']=expand_state
        #---output lines---
        self.driver_verbatim(lines)

    #-------------------
    def dump(self,prompt):
        for x in self.lines:
            debug(prompt,"line=%s" % string.rstrip(x))
    #===driver stubs==================
    def driver_reset(self):pass
    def driver_center(self):
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line):break                
    def driver_head(self,label,level,name):pass
    def driver_head_numbered(self,label,level,name):pass
    def driver_index(self):pass
    def driver_line(self,line): return line
    def driver_list(self,marker):pass
    def driver_para(self):
        self.rawout("\n")
    def driver_quote(self):pass
    def driver_slide_begin(self,title):pass
    def driver_slide_end(self):pass
    def driver_table_begin(self,caption):pass
    def driver_table_row_begin(self):pass
    def driver_table_cell_begin(self,width=None):pass
    def driver_table_cell_end(self):pass
    def driver_table_row_end(self):pass
    def driver_table_end(self,caption):pass
    def driver_texmath(self,lines):pass
    def driver_toc(self):pass
    def driver_verbatim(self,lines):pass
    def driver_verbatim_line(self,line):
        return line
    def driver_xref(self,line): return line
#==main=======================

if __name__=='__main__':
   ftn="main"
   pass



