#/* Pdx/Html.py 
# * Copyright (C) 1999  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */

__version__='0.4'

import sys,os,time,re,string,cgi
import Base

#revisions:
#1999-09-24  Harry George    reimplement from perl version

def debug(ftn,txt):
    sys.stdout.write("Pdx.Html."+ftn+":"+txt+"\n")
    sys.stdout.flush()
#====================================
def TT_fix(m):
    """
    Given a C<...> which becomes <TT>...</TT>,
    replace leading spaces in the contexts with hardspaces
    """
    oldstr=m.group(1)
    newstr='<TT>'
    for i in range(len(oldstr)):
        if oldstr[i]==' ':
            newstr=newstr+'&nbsp;'
        else:
            newstr=newstr+oldstr[i:]
            break   
    newstr=newstr+'</TT>'
    return newstr
#====================================
class Html (Base.Base):
    classname="Html"
    #------------------
    def __init__(self):
        Base.Base.__init__(self)
        self.driver="html"
        self.extension=".html"
    #------------------
    def __del__(self):
        Base.Base.__del__(self)
    #--------------------
    def driver_center(self):
        self.rawout('<CENTER>')
        self.process_file()
        self.nextline()  #past the =end
        self.rawout('</CENTER>')
    #--------------------
    def driver_head(self,label,level,name):
        if self._pass==1:
            data=[level,name,label]
            self._toc.append(data)
        self.rawout('\n<H%s><A NAME="%s">' % (level,label))
        self.out(name)
        self.rawout("</A></H"+level+">\n")
    #--------------------
    def driver_head_numbered(self,label,level,name):
        """
        Given stack of headings with [[1],[1,1],[1,2],[2]]
        get the current head, and build the label.
        Maintain stack.
        """        
        ftn="driver_head_numbered"
        level=int(level)
        heads=self.headstack
        
        #---fix the stack---
        while 1:
            if len(heads)==0:
                #debug(ftn,"fresh stack")
                heads.append([1])
                break
            
            top=heads[-1]
            toplevel=len(top)
            #debug(ftn,"level=%i,heads=%s" % (level,`heads`))
            
            if level==toplevel:
                #debug(ftn,"new head at same level")
                num=top[-1]
                num=num+1
                top[-1]=num
                heads[-1]=top
                break
            
            elif level < toplevel:
                #debug(ftn,"pop to parent,then add")
                while level < len(heads[-1]):
                    heads.pop()
            
            elif level > toplevel:
                #debug(ftn,"add a child")
                newtop=[]
                for x in top:
                    newtop.append(x)
                newtop.append(1)
                heads.append(newtop)
                break
            
        #---build the new label---
        newtop=heads[-1]
        label=""
        for x in newtop:
            label=label+`x`+'.'

        name=label+" "+name
        #debug(ftn,"name=%s" % name)

        #---report result----
        if self._pass==1:
            #---append to table of contents---
            data1=[level,name,label]
            self._toc.append(data1)
        self.rawout('\n<H%i><A NAME="%s">' %(level,label))
        self.out(name)
        self.rawout("</A></H%i>\n" % level)
            
    #--------------------
    def driver_index(self):
        ftn="driver_index"
        #debug(ftn,"begin with index="+`self._index`)
        if len(self._index)==0: return
        self._index.sort()
        self.do_head("1","Index")
        self.rawout("\n<UL>")
        currterm=''
        term=label=None
        for data in self._index:
            term,label=string.split(data,':')
            if term != currterm:
                currterm=term
                self.out("\n<LI>"+currterm)
                self.rawout(' <A HREF="#%s">%s</A>' % (label,label))
        self.rawout("\n</UL>\n")
            
    #--------------------
    def driver_line(self,line):
        ftn = "driver_line"
        #---text markups---
        line=self.B_pat.sub('<B>\\1 </B>',line)
        line=self.I_pat.sub('<I>\\1 </I>',line)
        line=self.C_pat.sub(TT_fix,line)
        line=self.U_pat.sub('<U>\\1 </U>',line)
        #---links---
        tmp=self.L_cgi_pat.search(line)
        if tmp!=None:
            #---check cgi tail for blanks---
            grp=tmp.group(2)
            grp=grp.replace(' ','%2B')
            line=self.L_cgi_pat.sub('<A HREF="\\1?%s">\\3</A>' % grp,line)
        else:
            #---non-cgi-link
            line=self.L_pat.sub('<A HREF="\\1">\\2</A>',line)
            
        line=self.N_pat.sub('<A NAME="\\1">',line)      
        #---escapes---
        line=self.lt_pat.sub('&lt;',line)
        line=self.gt_pat.sub('&gt;',line)
        line=self.amp_pat.sub('&amp;',line)
        line=self.quot_pat.sub('&quot;',line)
        line=self.lb_pat.sub('#',line)

        #debug(ftn,"cooked line="+line)
        return line

    #--------------------
    def driver_list(self,marker):
        ftn = "driver_list"
        #debug(ftn,"begin with "+marker)
        before_map={
            '*':'<UL>',
            'N':'<UL>',
            '1':'<OL TYPE="1">',
            'a':'<OL TYPE="a">',
            'A':'<OL TYPE="A">',
            'i':'<OL TYPE="i">',
            'I':'<OL TYPE="I">',
            's':'<OL TYPE="square">',
            'c':'<OL TYPE="circle">',
            '[':'<DL>',
            }
        after_map={
            '*':'</UL>',
            'N':'</UL>',
            '1':'</OL>',
            'a':'</OL>',
            'A':'</OL>',
            'i':'</OL>',
            'I':'</OL>',
            's':'</OL>',
            'c':'</OL>',
            '[':'</DL>',
            }
        before=before_map[marker]
        after=after_map[marker]
        #debug(ftn,"before="+before+", after="+after)
        self.rawout(before)
        
        lists=self.liststack
        if marker=='N':
            #---fully numbered list, starting at 0---
            lists.append(0)
            numbered_p=1
        else:
            numbered_p=0
            
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line) : break  #normal lists
            if self.back_pat.search(line): break  #pod-style over...back
            m=self.item_pat.search(line)
            if m:
                txt=m.group(1)

                if numbered_p:
                    lists[-1]=lists[-1]+1
                    numstr=''
                    for item in lists:
                        numstr=numstr+`item`+'.'
                    self.out('\n<LI>%s %s\n' % (numstr,txt))
                    
                elif marker == '[':
                    self.out('\n<DT><STRONG>%s</STRONG><DD>' % (txt,))
                
                else:
                    self.out('\n<LI>%s\n' % (txt,))
            else:
                self.ndx=self.ndx-1
                self.process_file()
        self.rawout(after)
        if numbered_p:
            lists.pop()
    #-------------------
    def driver_para(self):
        ftn = "driver_para"
        #debug(ftn,"begin")
        self.rawout("<p>\n")
    #--------------------
    def driver_quote(self):
        self.rawout('<BLOCKQUOTE>')
        self.process_file()
        self.nextline()  #past the =end
        self.rawout('</BLOCKQUOTE>')
    #--------------------
    def driver_table_begin(self,caption):
        if self.table_grid_p:
            self.rawout('<TABLE BORDER CELLSPACING=2 CELLPADDING=2>')
        else:
            self.rawout('<TABLE CELLSPACING=2 CELLPADDING=2>')
        self.rawout('<CAPTION ALIGN="TOP">')
        self.out(caption)
        self.rawout('</CAPTION >'+"\n")
        
    #--------------------
    def driver_table_end(self,caption):
        self.rawout('</TABLE>'+"\n")
    #--------------------
    def driver_table_cell_begin(self,width=None):
        self.rawout('<TD VALIGN="TOP" ')
        if width:
            self.rawout('WIDTH="'+`width`+'%"')
        self.rawout('>')
    #--------------------
    def driver_table_cell_end(self):
        self.rawout('</TD>')
    #--------------------
    def driver_table_row_begin(self):
        self.rawout('<TR>')
    #--------------------
    def driver_table_row_end(self):
        self.rawout('</TR>\n')
        
    #--------------------
    def driver_texmath(self,lines):
        name=self.data['_basename'] + '_' + self.genlabel()
        self.do_texmath_png(name,lines)
        self.rawout('<IMG SRC="'+name+'.png" ALT="'+name+'.png">')
    #--------------------
    def driver_toc(self):
        ftn = "driver_toc"
        self.rawout(r'<H1><A NAME="#toc">Table of Contents</A></H1>'+"\n")
        self.rawout("<UL>\n")
        currlevel=1
        level=name=label=None
        for entry in self._toc:
            level,name,label=entry
            level=int(level)
            while currlevel < level:
                self.rawout("<UL>\n")
                currlevel=currlevel+1
            while currlevel > level:
                self.rawout(" </UL>\n")
                currlevel=currlevel-1
            self.rawout(r'<LI><A HREF="#'+str(label)+r'"> ')
            self.out(name+"</A>\n")
        while currlevel > 0:
            self.rawout(" </UL>")
            currlevel=currlevel-1
            
    #-------------------------
    def driver_verbatim(self,lines):
        ftn="driver_verbatim"
        #debug(ftn,"lines="+`lines`)
        self.rawout(string.join(['\n<P','RE>\n'],''))
        for line in lines:
            line=cgi.escape(line,quote=1)
            self.rawout(line)
        self.rawout(string.join(['</P','RE>\n'],''))

##             line=self.amp_pat.sub('&amp;',line)
##             line=self.lt_pat.sub('&lt;',line)
##             line=self.gt_pat.sub('&gt;',line)
##             line=self.quot_pat.sub('&quot;',line)
    #---------------------
    def driver_xref(self,term):
        ftn = "do_xref"
        label=self.genlabel()
        if self._pass==1:
            data=string.join([term,label],":")
            self._index.append(data)
            self._refs[term]=label
        xref_txt = ' <A NAME="'+label+'"></A>'
        return xref_txt

#==main=======================
def test(basename):
    ftn="test"
    sys.stderr.write(basename+"...")
    olddir=os.getcwd()
    os.chdir('../test')
    p=Html()
    pdxname=basename+'.pdx'
    p.parse([pdxname])
    sys.stderr.write(p.data['desc']+"...")
    ndx=basename[-2:]
    testname="test"+ndx+".html"
    oraclename="oracle"+ndx+".html"
    result=commands.getoutput("diff "+testname +" "+oraclename)
    resultlen=len(result)
    if resultlen==0:
        sys.stderr.write("passed\n")
    else:
        sys.stderr.write("failed:\n"+result+"\n")
    os.chdir(olddir)
    del p

def testloop(basename):
    olddir=os.getcwd()
    os.chdir('testdata')
    pdxname=basename+'.pdx'
    for i in range(1,10):
        p=Html()
        p.parse([pdxname]) 
        del p
    os.chdir(olddir)

if __name__=='__main__':
   ftn="main"
   import commands,profile
   #---get local Base and Html---
   del Base
   sys.path.insert(0,'.')
   sys.path.insert(0,'..')
   import Base

   #profile.run('testloop("test01")')
   test("test01")
   test("test02")
   test("test03")
   test("test04")
   test("test05")
   test("test06")
   test("test07")
   test("test08")
   test("test10")

