#!/usr/bin/env python

"""
NAME:
    PyPt.py  

SYNOPSIS:
    PyPt.py [options] filename

DESCRIPTION:
    Reportlab Pythonpoint backend for Pdx.

OPTIONS:
    -h,--help        this message
    -v,--version     version
    -d,--debug       turn on debug messages

EXAMPLES:
    1. As standalone
        PyPt.py -d myfile
    2. As library
        import PyPt
        ...

#Copyright (C) 2002  Harry George hgg9140@seanet.com
#http://www.seanet.com/~hgg9140
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


"""
#===imports======================
import sys,os,getopt,time
import Base

#===globals======================
modname="PyPt"
__version__="0.1"

#--option args--
debug_p=0

#---positional args, default is empty---
pargs=[]    

#---other---

#===utilities====================
def msg(txt):
    sys.stdout.write(txt)
    sys.stdout.flush()

def debug(ftn,txt):
    if debug_p:
        sys.stdout.write("%s.%s:%s\n" % (modname,ftn,txt))
        sys.stdout.flush()

def fatal(ftn,txt):
    msg="%s.%s:FATAL:%s\n" % (modname,ftn,txt)
    raise SystemExit, msg
 
def usage():
    print __doc__
#========================================================

class PyPt (Base.Base):
    classname="PyPt"
    def __init__(self):
        Base.Base.__init__(self)
        self.driver="pypt"
        self.extension=".xml"

        self.in_slide_p=0
        self.slide_title=''
        self.maxlines=20  # num of lines allowed per slide
        self.line_cnt=0  # num of lines so far in slide
        self.cont_cnt=0  # num of continuations on this slide

        self.in_para_p=0
        self.para_style='BodyText'
    #------------------
    def __del__(self):
        Base.Base.__del__(self)
    #--------------------
    def driver_center(self):
        self.para_style='Centered'
        self.driver_para()
        self.process_file()
        self.nextline()  #past the =end
        self.para_style='BodyText'
        self.driver_para()
    #--------------------
    def driver_head(self,label,level,name):
        name=self.driver_line(name)
        headname='Heading'+`int(level)`
        self.rawout('<para style="'+headname+'">'+name+'</para>\n')        
        self.driver_para()
    #--------------------
    def driver_head_end(self,old_level=None):
        self.driver_para_end()
    #--------------------
    def driver_head_numbered(self,label,level,name):
        ftn="driver_head_numbered"
        self.driver_para_end()
        self.driver_head(label,level,name)
    #--------------------
    def driver_index(self):
        return

    
        if self.data['index_p']==0: return
        if len(self._index)==0: return
        self._index.sort()
        self.rawout('\n<INDEX><TITLE>Index</TITLE>\n');
        currterm=''
        term=label=None
        for data in self._index:
            term,label=data
            if term != currterm:
                currterm=term
                self.rawout('<INDEXENTRY><PRIMARYIE '
                 + 'LINKENDS="' + label + '">' 
                 + currterm + '</PRIMARYIE></INDEXENTRY>\n');

        self.rawout("</INDEX>\n")
            
    #--------------------
    def driver_line_increment(self,inc=1):
        """auto slide continuation"""
        self.line_cnt+=inc
        if self.in_slide_p and self.line_cnt > self.maxlines:
            self.line_cnt=0
            self.driver_slide_end()
            self.driver_slide_begin('',cont_p=1)

    #--------------------
    def driver_line(self,line):
        ftn = "driver_line"

        self.driver_line_increment()
        
        #---text markups---
        line=self.break_pat.sub('</para><para style="%s">'  \
                                % self.para_style,line)
        line=self.B_pat.sub('<b>\\1</b>',line)
        line=self.I_pat.sub('<i>\\1</i>',line)
        line=self.C_pat.sub('<LITERAL>\\1</LITERAL>',line)
        line=self.U_pat.sub('<u>\\1</u>',line)
        #---links---
        line=self.N_pat.sub('<ANCHOR ID="\\1">',line)
        line=self.L_hash_pat.sub('<LINK ID="\\1">\\2</LINK>',line)
        line=self.L_pat.sub('<A HERF="\\1">\\2</A>',line)
        #---escapes---
        line=self.lt_pat.sub('(lt)',line)
        line=self.gt_pat.sub('&gt;',line)
        line=self.amp_pat.sub('(amp)',line)
        line=self.quot_pat.sub('&quot;',line)
        line=self.lb_pat.sub('#',line)

        #debug(ftn,"cooked line="+line)
        return line
        
    #--------------------
    def driver_list(self,marker):
        ftn = "driver_list"
        #debug(ftn,"begin with "+marker)
        self.driver_para_end()
        style_map={
            '*':'Bullet',
            '1':'Bullet',
            'a':'Bullet',
            'A':'Bullet',
            '[':'BodyText',
            'N':'Bullet',
            }

        style=style_map[marker]
        while self.morelines():
            line=self.nextline()
            if self.end_pat.search(line) : break  #normal lists
            if self.back_pat.search(line): break  #pod-style over...back
            m=self.item_pat.search(line)
            if m:
                if marker == '[':
                    self.driver_para(style)
                    self.out('\n%s\n' % (m.group(1)))
                    self.para_style='Indent'
                    self.driver_para()
                else:
                    self.driver_para(style)
                    self.out('\n%s\n' % (m.group(1)))
                    self.para_style='BodyText'
                    self.driver_para()
                    

            elif self.end_pat.search(line) : break
            elif self.back_pat.search(line): break
            else:
                self.ndx=self.ndx-1
                self.process_file()
                if marker == '[':
                    pass
                else:
                    pass
    #-------------------
    def driver_para(self,style=None):
        ftn = "driver_para"
        if not self.in_slide_p:return
        self.driver_para_end()
        if not style:
            style=self.para_style
        self.rawout('<para style="%s">\n' % style)
        self.in_para_p=1

    #--------------------
    def driver_para_end(self,level=0):
        if not self.in_slide_p:return
        if self.in_para_p==1:
            self.rawout('</para>\n')
        self.in_para_p=0
    #--------------------
    def driver_quote(self):
        self.para_style='Indent'
        self.driver_para()
        self.process_file()
        self.nextline()  #past the =end
        self.para_style='BodyText'
        self.driver_para()
    #--------------------
    def driver_slide_begin(self,title,cont_p=0):
        self.driver_slide_end()
        
        if cont_p:
            self.cont_cnt+=1
            title=self.slide_title + " (cont. %i)" % self.cont_cnt
        else:
            self.cont_cnt=0
            self.slide_title=title
        self.in_slide_p=1
        self.line_cnt=0
        label=self.genlabel()
        self.rawout('<slide id="%s" title="%s" ' % (label,title))
        self.rawout('>\n')
        self.rawout('<frame x="120" y="72" width="700" height="468"' \
                        ' leftmargin="36" rightmargin="36">\n')
        self.driver_para()
    #--------------------
    def driver_slide_end(self):
        self.driver_para_end()
        if self.in_slide_p:
            self.rawout("</frame></slide>\n\n")
        self.in_slide_p=0
    #--------------------
    def driver_table_begin(self,caption):
        widths = self.table_colwidths
        numcols=self.table_numcols
        self.rawout('<TABLE ')
        if self.table_grid_p:
            self.rawout('FRAME="ALL" COLSEP="1" ROWSEP="1">')
        else:
            self.rawout('FRAME="NONE" COLSEP="0" ROWSEP="0">')
        self.rawout("\n<TITLE>"+caption+"</TITLE>")
        self.rawout("\n"+r'<TGROUP COLS="' + `numcols` + r'">')
        for i in range(numcols):
            width=widths[i]
            self.rawout("\n"
                               + r'<COLSPEC COLNUM="' + `i+1` + r'"'
                               + r' COLWIDTH="0.' + width + r'*">')
        self.rawout('\n<TBODY>\n')
        
            
        
    #--------------------
    def driver_table_end(self,caption):
        self.rawout('</TBODY></TGROUP></TABLE>\n')

    #--------------------
    def driver_table_row(self,cells):
        numcols=self.table_numcols
        self.rawout('<ROW>')
        for i in range(numcols):
            cell=cells[i]
            self.rawout('<ENTRY>')
            self.out(cell)
            self.rawout('</ENTRY>')
        self.rawout('</ROW>'+"\n")
        
    #--------------------
    def driver_texmath(self,lines):
        name=self.data['_basename'] + '_' + self.genlabel()
        self.do_texmath_png(name,lines)
        self.rawout('<GRAPHIC FILEREF="'+name+'.png" FORMAT="'+name+'.png">')
    #--------------------
    def driver_toc(self):
        ftn="driver_toc"

    #--------------------
    def driver_verbatim(self,lines):
        self.driver_para_end()
        self.rawout("<prefmt style='Code'><![CDATA[\n")
        for line in lines:
            line=self.amp_pat.sub('&amp;',line)
            line=self.break_pat.sub("&lt;BR&gt;",line)
            line=self.lt_pat.sub('&lt;',line)
            line=self.gt_pat.sub('&gt;',line)
            line=self.quot_pat.sub('&quot;',line)
            self.rawout(line)
        self.rawout("]]>\n</prefmt>\n")
        self.driver_para()

    #---------------------
    def driver_xref(self,term):
        ftn = "do_xref"
        label=self.genlabel()
        if self._pass==1:
            data=(term,label)
            self._index.append(data)
            self._refs[term]=label
        xref_txt = '<INDEXTERM ID="'+label+'"><PRIMARY>'+term+'</PRIMARY></INDEXTERM>'
        return xref_txt

#=======================================
if __name__=='__main__':
    pass


