#!/usr/bin/env python
# basic_tests.py 
#    Copyright (C) 2001  Harry George hgg9140@seanet.com
#    http://www.seanet.com/~hgg9140
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Library General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


"""
NAME:
    basic_tests.py  

SYNOPSIS:
    basic_tests.py

DESCRIPTION:
    Run basic, file level tests for XML Schema

OPTIONS:
    -h,--help        this message
    -v,--version     version

EXAMPLES:
    1. As standalone
         basic_tests.py 
"""
#===imports======================
import sys,os,getopt,time,string,filecmp
import unittest

sys.path.insert(0,"..")
from Pdx import Base,Html,Latex,Seminar,Docbook,PyPt,Rest

#===globals======================
modname="basic_tests"
__version__="0.1"

#--option args--

#---positional args, default is empty---
#---other----------------
#===utilities====================
def debug(ftn,txt):
    sys.stdout.write(string.join([modname,'.',ftn,':',txt,'\n'],''))
    sys.stdout.flush()

def fatal(ftn,txt):
    msg=string.join([modname,'.',ftn,':FATAL:',txt,'\n'],'')
    raise SystemExit, msg
 
def usage():
    print __doc__



#====================================
class BasicTestCase(unittest.TestCase):
    #------------------------
    def setUp(self):
        self.olddir=os.getcwd()
        os.chdir('testdata')
        self.base=Base.Base()
        self.html=Html.Html()
        self.latex=Latex.Latex()
        #self.seminar=Latex.Latex()
        self.seminar=Seminar.Seminar()
        self.docbook=Docbook.Docbook()
        self.pypt=PyPt.PyPt()
        self.rest=Rest.Rest()
     #------------------------
    def tearDown(self):
        del self.base
        del self.html
        del self.latex
        del self.seminar
        del self.docbook
        del self.pypt
        del self.rest
        os.chdir(self.olddir)

    #=========================
    # Base
    #=========================
    #------------------------
    def mk_base(self,ndx):
        p=self.base
        pdxname   ="test"  +ndx+'.pdx'
        assert p.parse([pdxname])

    #-----------------------
    def checkHello_base(self):
        "base: 01 'hello, world'"
        self.mk_base("01")
    #-----------------------
    def checkHeadings_base(self):
        "base: 02 headings"
        self.mk_base("02")
    #-----------------------
    def checkLists_base(self):
        "base: 03 lists"
        self.mk_base("03")
    #-----------------------
    def checkTables_base(self):
        "base: 04 tables"
        self.mk_base("04")
    #-----------------------
    def checkIncludes_base(self):
        "base: 05 includes"
        self.mk_base("05")
    #-----------------------
    def checkLinks_base(self):
        "base: 06 links"
        self.mk_base("06")
    #-----------------------
    def checkQuotes_base(self):
        "base: 07 quotes"
        self.mk_base("07")
    #-----------------------
    def checkStyles_base(self):
        "base: 08 styles"
        self.mk_base("08")
    #-----------------------
    def checkSlides_base(self):
        "base: 09 slides"
        self.mk_base("09")
    #-----------------------
    def checkMath_base(self):
        "base: 10 math"
        self.mk_base("10")
    #-----------------------
    def checkBigFile_base(self):
        "base: 11 big file"
        self.mk_base("11")

    #===========================
    # Html
    #===========================
    #------------------------
    def mk_html(self,ndx):
        p=self.html
        pdxname   ="test"  +ndx+'.pdx'
        testname  ="test"  +ndx+'.html'
        oraclename="oracle"+ndx+'.html'
        p.parse([pdxname])
        assert filecmp.cmp(testname,oraclename)

    #-----------------------
    def checkHello_html(self):
        "html: 01 'hello, world'"
        self.mk_html("01")
    #-----------------------
    def checkHeadings_html(self):
        "html: 02 headings"
        self.mk_html("02")
    #-----------------------
    def checkLists_html(self):
        "html: 03 lists"
        self.mk_html("03")
    #-----------------------
    def checkTables_html(self):
        "html: 04 tables"
        self.mk_html("04")
    #-----------------------
    def checkIncludes_html(self):
        "html: 05 includes"
        self.mk_html("05")
    #-----------------------
    def checkLinks_html(self):
        "html: 06 links"
        self.mk_html("06")
    #-----------------------
    def checkQuotes_html(self):
        "html: 07 quotes"
        self.mk_html("07")
    #-----------------------
    def checkStyles_html(self):
        "html: 08 styles"
        self.mk_html("08")
    #-----------------------
    def checkSlides_html(self):
        "html: 09 slides"
        #self.mk_html("09")
        pass
    #-----------------------
    def checkMath_html(self):
        "html: 10 math"
        self.mk_html("10")
    #-----------------------
    def checkBigFile_html(self):
        "html: 11 big file"
        self.mk_html("11")

    #===========================
    # Latex
    #===========================
    #------------------------
    def mk_latex(self,ndx):
        p=self.latex
        pdxname   ="test"  +ndx+'.pdx'
        testname  ="test"  +ndx+'.tex'
        oraclename="oracle"+ndx+'.tex'
        p.parse([pdxname])
        assert filecmp.cmp(testname,oraclename)
    #-----------------------
    def checkHello_latex(self):
        "latex: 01 'hello, world'"
        self.mk_latex("01")
    #-----------------------
    def checkHeadings_latex(self):
        "latex: 02 headings"
        self.mk_latex("02")
    #-----------------------
    def checkLists_latex(self):
        "latex: 03 lists"
        self.mk_latex("03")
    #-----------------------
    def checkTables_latex(self):
        "latex: 04 tables"
        self.mk_latex("04")
    #-----------------------
    def checkIncludes_latex(self):
        "latex: 05 includes"
        self.mk_latex("05")
    #-----------------------
    def checkLinks_latex(self):
        "latex: 06 links"
        self.mk_latex("06")
    #-----------------------
    def checkQuotes_latex(self):
        "latex: 07 quotes"
        self.mk_latex("07")
    #-----------------------
    def checkStyles_latex(self):
        "latex: 08 styles"
        self.mk_latex("08")
    #-----------------------
    def checkSlides_latex(self):
        "latex: 09 slides"
        #self.mk_latex("09")
        pass
    #-----------------------
    def checkMath_latex(self):
        "latex: 10 math"
        self.mk_latex("10")
    #-----------------------
    def checkBigFile_latex(self):
        "latex: 11 bigfile"
        self.mk_latex("11")
    #===========================
    # Seminar
    #===========================
    #------------------------
    def mk_seminar(self,ndx):
        p=self.seminar
        basename  ="test"  +ndx
        pdxname   =basename+'.pdx'
        testname  =basename+'.tex'
        #---use "stex" to avoid conflict with normal latex---
        oraclename="oracle"+ndx+'.stex'
        p.parse([pdxname])
        assert filecmp.cmp(testname,oraclename)
    #-----------------------
    #---we only use test09, which includes all the others---
    #-----------------------
    def checkHello_seminar(self):
        "seminar: 01 'hello, world'"
        pass
    #-----------------------
    def checkHeadings_seminar(self):
        "seminar: 02 headings"
        pass
    #-----------------------
    def checkLists_seminar(self):
        "seminar: 03 lists"
        pass
    #-----------------------
    def checkTables_seminar(self):
        "seminar: 04 tables"
        pass
    #-----------------------
    def checkIncludes_seminar(self):
        "seminar: 05 includes"
        pass
    #-----------------------
    def checkLinks_seminar(self):
        "seminar: 06 links"
        pass
    #-----------------------
    def checkQuotes_seminar(self):
        "seminar: 07 quotes"
        pass
    #-----------------------
    def checkStyles_seminar(self):
        "seminar: 08 styles"
        pass
    #-----------------------
    def checkSlides_seminar(self):
        "seminar: 09 slides"
        self.mk_seminar("09")
    #-----------------------
    def checkMath_seminar(self):
        "seminar: 10 math"
        pass

    #===========================
    # Docbook
    #===========================
    #------------------------
    def mk_docbook(self,ndx):
        p=self.docbook
        pdxname   ="test"  +ndx+'.pdx'
        testname  ="test"  +ndx+'.sgml'
        oraclename="oracle"+ndx+'.sgml'
        p.parse([pdxname])
        assert filecmp.cmp(testname,oraclename)
    #-----------------------
    def checkHello_docbook(self):
        "docbook: 01 'hello, world'"
        self.mk_docbook("01")
    #-----------------------
    def checkHeadings_docbook(self):
        "docbook: 02 headings"
        self.mk_docbook("02")
    #-----------------------
    def checkLists_docbook(self):
        "docbook: 03 lists"
        self.mk_docbook("03")
    #-----------------------
    def checkTables_docbook(self):
        "docbook: 04 tables"
        self.mk_docbook("04")
    #-----------------------
    def checkIncludes_docbook(self):
        "docbook: 05 includes"
        self.mk_docbook("05")
    #-----------------------
    def checkLinks_docbook(self):
        "docbook: 06 links"
        self.mk_docbook("06")
    #-----------------------
    def checkQuotes_docbook(self):
        "docbook: 07 quotes"
        self.mk_docbook("07")
    #-----------------------
    def checkStyles_docbook(self):
        "docbook: 08 styles"
        self.mk_docbook("08")
    #-----------------------
    def checkSlides_docbook(self):
        "docbook: 09 slides"
        self.mk_docbook("09")
    #-----------------------
    def checkMath_docbook(self):
        "docbook: 10 math"
        self.mk_docbook("10")


    #===========================
    # PyPt
    #===========================
    #------------------------
    def mk_pypt(self,ndx):
        p=self.pypt
        pdxname   ="test"  +ndx+'.pdx'
        testname  ="test"  +ndx+'.xml'
        oraclename="oracle"+ndx+'.xml'
        p.parse([pdxname,'--slides'])
        assert filecmp.cmp(testname,oraclename)

    #-----------------------
    def checkHello_pypt(self):
        "pypt: 01 'hello, world'"
        self.mk_pypt("01")
    #-----------------------
    def checkHeadings_pypt(self):
        "pypt: 02 headings"
        self.mk_pypt("02")
    #-----------------------
    def checkLists_pypt(self):
        "pypt: 03 lists"
        self.mk_pypt("03")
    #-----------------------
    def checkTables_pypt(self):
        "pypt: 04 tables"
        self.mk_pypt("04")
    #-----------------------
    def checkIncludes_pypt(self):
        "pypt: 05 includes"
        self.mk_pypt("05")
    #-----------------------
    def checkLinks_pypt(self):
        "pypt: 06 links"
        self.mk_pypt("06")
    #-----------------------
    def checkQuotes_pypt(self):
        "pypt: 07 quotes"
        self.mk_pypt("07")
    #-----------------------
    def checkStyles_pypt(self):
        "pypt: 08 styles"
        self.mk_pypt("08")
    #-----------------------
    def checkSlides_pypt(self):
        "pypt: 09 slides"
        self.mk_pypt("09")
    #-----------------------
    def checkMath_pypt(self):
        "pypt: 10 math"
        self.mk_pypt("10")

    #===========================
    # Rest
    #===========================
    #------------------------
    def mk_rest(self,ndx):
        p=self.rest
        pdxname   ="test"  +ndx+'.pdx'
        testname  ="test"  +ndx+'.rst'
        oraclename="oracle"+ndx+'.rst'
        p.parse([pdxname])
        assert filecmp.cmp(testname,oraclename)

    #-----------------------
    def checkHello_rest(self):
        "rest: 01 'hello, world'"
        self.mk_rest("01")
    #-----------------------
    def checkHeadings_rest(self):
        "rest: 02 headings"
        self.mk_rest("02")
    #-----------------------
    def checkLists_rest(self):
        "rest: 03 lists"
        self.mk_rest("03")
    #-----------------------
    def checkTables_rest(self):
        "rest: 04 tables"
        self.mk_rest("04")
    #-----------------------
    def checkIncludes_rest(self):
        "rest: 05 includes"
        self.mk_rest("05")
    #-----------------------
    def checkLinks_rest(self):
        "rest: 06 links"
        self.mk_rest("06")
    #-----------------------
    def checkQuotes_rest(self):
        "rest: 07 quotes"
        self.mk_rest("07")
    #-----------------------
    def checkStyles_rest(self):
        "rest: 08 styles"
        self.mk_rest("08")
    #-----------------------
    def checkSlides_rest(self):
        "rest: 09 slides"
        #self.mk_rest("09")
        pass
    #-----------------------
    def checkMath_rest(self):
        "rest: 10 math"
        self.mk_rest("10")

#=====================================
def suite():
    suite1 = unittest.TestSuite()
    cases=[
           'Hello',
           'Headings',
           'Lists',
           'Tables',
           'Includes',
           'Links',
           'Quotes',
           'Styles',
           'Slides',
           'Math',
           #'BigFile',
           ]
    backends=[
              'base',
              'html',
              'latex',
              #'seminar',
              #'docbook',
              #'pypt',
              'rest',
              ]
    
    for b in backends:
        for c in cases:
            suite1.addTest(BasicTestCase("check%s_%s" % (c,b)))
    return suite1

    

#=============================
def main():
    unittest.main(defaultTest='suite')
    
#-------------------------
if __name__ == '__main__':
    ftn = "main"
    opts,pargs=getopt.getopt(sys.argv[1:],'hv',
                 ['help','version','aa','bb='])
    for opt in opts:
        if opt[0]=='-h' or opt[0]=='--help':
            print modname+": version="+__version__
            usage()
            sys.exit(0)
        elif opt[0]=='-v' or opt[0]=='--version':
            print modname+": version="+__version__
            sys.exit(0)
        elif opt[0]=='--aa':
            opt_a=1
        elif opt[0]=='--bb':
            opt_b=opt[1]

    #---make the object and run it---
    main()

#===Revision Log===
#Created by mkpythonproj:
#2001-02-10  Harry George  
#

#$Log: basic_tests.py$
